#ifndef CAFFE_GRADIENT_REVERSAL_LAYER_
#define CAFFE_GRADIENT_REVERSAL_LAYER_

#include "caffe/blob.hpp"
#include "caffe/layer.hpp"
#include "caffe/proto/caffe.pb.h"

#include "caffe/layers/neuron_layer.hpp"

namespace caffe {

/**
 * @brief Identity transformation in the forward pass,
 *        inverting gradient during backprop
 */
template <typename Dtype>
class GradientReversalLayer : public NeuronLayer<Dtype> {
 public:
  explicit GradientReversalLayer(const LayerParameter& param)
      : NeuronLayer<Dtype>(param) {}


  virtual void LayerSetUp(const vector<Blob<Dtype>*>& bottom,
                          const vector<Blob<Dtype>*>& top);

  virtual inline const char* type() const { return "GradientReversal"; }

 protected:

  virtual void Forward_cpu(const vector<Blob<Dtype>*>& bottom,
      const vector<Blob<Dtype>*>& top);
  virtual void Forward_gpu(const vector<Blob<Dtype>*>& bottom,
      const vector<Blob<Dtype>*>& top);

  virtual void Backward_cpu(const vector<Blob<Dtype>*>& top,
      const vector<bool>& propagate_down, const vector<Blob<Dtype>*>& bottom);
  virtual void Backward_gpu(const vector<Blob<Dtype>*>& top,
      const vector<bool>& propagate_down, const vector<Blob<Dtype>*>& bottom);

};

}  // namespace caffe

#endif  // CAFFE_GRADIENT_REVERSAL_LAYER_
